/*
 * Motf - A maximal motif miner for a collection of sequences
 * Copyright 2005, Hiroki Arimura, Takeaki Uno, all rights reserved.
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Efficient enumeration of maximal motifs in a sequence,
 * Hiroki Arimura and Takeaki Uno, Manuscript, Division of
 * Computer Science, * IST, Hokkaido University, 9 April 2005;
 * Modified 18 April 2005.  
 * <http:>
 *
 * This software may be used freely for any purpose. However, when
 * distributed, the original source must be clearly stated, and, 
 * when the source code is distributed, the copiright notice must be
 * retained and any alterations in the code must be clearly marked. 
 * No warranty is given regarding the quality of this software.
 */

package motf; 
import motf.*;
import java.io.*;
import java.util.*;

/**
 * MotfMiner.java
 *
 * Created: 08 June 2005
 * @author <a href="mailto:arim@ist.hokudai.ac.jp">Hiroki Arimura</a>
 * @version $Revision: 1.9 $
 */

public abstract class MotFinder {

    /** Java  DEBUG ͡ */
    public static final boolean useDebug = false;
    public static final boolean useDebugOccPool = false;
    public static final boolean useDebugExpand  = false;
	public static final boolean useDebugOutput = false;
	public static final boolean useDebugMinsup = false;

	public final String versionString = "MotFinder - Here the name of routine comes";

    /** ѥΥ¤ʤȤ򼨤 */
    public static boolean useMaxPatSize = false; 

	/** ѥȯоݤȤʤʸ񽸹 */
	DocSet docSet = null; 

	/** ʸ񽸹ʸиɽ */
	OccPool occPool = null; 

    /** ȯѥΥꥹȤǼѥ󥻥åȡ */
    PatSet patSet = null;

	/** ѥɾؿ׻륪֥ */
	Scorer scorer = null; 

    /** ѥκ祵
	 *  ǥեȤ¤ʤɽ魯 Integer.MAX_VALUE = 2^{31}-1*/
    int maxPatSize = Integer.MAX_VALUE; 

    /** ѥκ祮åĹ*/
    int maxGapLen = -1; 

	/** õѥ */
	int numberOfExpansion = 0;

    /** å٤Ȥʸ٤Ѥ뤫ɤ򼨤 */
	public boolean useDocCount = false;

    /** åƥѥȰ˰٤ʸ٤뤫 */
	public static boolean usePrintCount 	= false; 

    /** åƥѥȰ˰٤ʸ٤뤫 */
	public static boolean usePrintOcc	= false;

    /** ׻β뤫ɤΥå*/
	public static boolean usePrintTrace = false; 

	/*
	 * Procedures
	 */

	public MotFinder(DocSet docSet) {
		this.docSet = docSet; 
	}

	public abstract String getVersionString(); 

	public void setScorer(Scorer scorer) {
		this.scorer = scorer; 
	}

	public void setMaxPatSize(int maxSize) {
		useMaxPatSize = true;
		maxPatSize = maxSize;
		Log.println("@log maxPatSize:\t"+maxSize);
	}

	public void setMaxGapLen(int maxGapLen) {
		this.maxGapLen = maxGapLen; 
		Log.println("@log maxGapLen:\t"+maxGapLen);
	}

	public void setUseDocCount(boolean flag) {
		this.useDocCount = flag; 
	}

    /** Motfᥤץ¹Ԥ롥ϺǾͤ͡
     */
    public PatSet run(double minSupRatio)
    {
        Log.println("@log Minimum_Support_Relative: "+minSupRatio);
		return run(docSet.convertSupRatioToAbsolute(minSupRatio));
	}

    /** Motfᥤץ¹Ԥ롥ϺǾͤ͡
     */
    public abstract PatSet run(int minSupCount); 

	//
	//Utilities
	//

	/** ĤäѥPatSetϿ롥
	 */
	void registerPattern(Pattern pattern, Occurrence occ) {
		//ѥ󤬸Ĥä
		this.patSet.add(pattern);
		if (this.usePrintCount) {
			pattern.setPosCount(occ.getPosCount());
			pattern.setDocCount(occ.getDocCount());
		}

		if (this.usePrintOcc) 
			pattern.setOccurrence(occ);

		if (useDebugOutput) Log.println("@log expand: enter\n"+
					"pattern "+pattern.toString());

	}

}

/* EOF */
