/*
 * Motf.java
 * Copyright 2005, Hiroki Arimura, Takeaki Uno, all rights reserved.
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Efficient enumeration of maximal motifs in a sequence,
 * Hiroki Arimura and Takeaki Uno, Manuscript, Division of
 * Computer Science, * IST, Hokkaido University, 9 April 2005;
 * Modified 18 April 2005.  
 * <http:>
 *
 * This software may be used freely for any purpose. However, when
 * distributed, the original source must be clearly stated, and, 
 * when the source code is distributed, the copiright notice must be
 * retained and any alterations in the code must be clearly marked. 
 * No warranty is given regarding the quality of this software.
 */

package motf; 
import motf.*;
import java.io.*;
import java.util.*;

/**
 * Classname.java
 *
 * Created: 08 June 2005
 * @author <a href="mailto:arim@ist.hokudai.ac.jp">Hiroki Arimura</a>
 * @version $Revision: 1.2 $
 */

public class PatternComparator implements Comparator<Pattern> {
	public int compare(Pattern pat1, Pattern pat2) { return 0; }; 
}

class PatternSizeComparator extends PatternComparator {
	public int compare(Pattern pat1, Pattern pat2) {
		int size1 = pat1.size();
		int size2 = pat2.size();
		return size1 - size2; 
	}
}

class PatternPosCountComparator extends PatternComparator {
	public int compare(Pattern pat1, Pattern pat2) {
		int size1 = pat1.getPosCount();
		int size2 = pat2.getPosCount();
		return size1 - size2; 
	}
}

class PatternDocCountComparator extends PatternComparator {
	public int compare(Pattern pat1, Pattern pat2) {
		int size1 = pat1.getDocCount();
		int size2 = pat2.getDocCount();
		return size1 - size2; 
	}
}

/* EOF */
